/**
 * Twitter API Port
 * https://github.com/mitzerh/Twitter-JS-Widget
 * MIT License. Copyright (c) 2011, Helcon Mabesa - http://www.hmabesa.com/
 */
(function($){
	
	var CONST = {
		feed: {
			url: "http://search.twitter.com/search.json",
			query: "?q=${search}&rpp=${rpp}&callback=?",
			type: "jsonp"
		},
		trend: {
			url: "http://api.twitter.com/1/trends/${format}.json",
			data: { exclude:"hashtags" },
			type: "jsonp"
		},
		cookie: {
			name: "TW_Proto",
			expires: 30 // in minutes
		},
		defaults: {
			resultsPerPage: 5, // default number of items
			refresh: 60, // in seconds
			minRefresh: 30, // in seconds - minimum refresh interval
			view: { type:"batch" }, // default view
			stackFreq: 3, // stack frequency
			stackPrimary: "full", // full or single primary rendering
			maxItems: 300 // number of maximum items to retain
		}
	};
	
	CONST.defaultMarkup = (function(){
		var html = [];
		html.push('<li${hideStyle} id="tid-${tweetId}">');
		html.push('<div class="user-img"><a href="http://twitter.com/${user}" target="_blank"><img alt="${user}" src="${imgSrc}" /></a></div>');
		html.push('<div class="content">');
		html.push('<p><span class="username"><a href="http://twitter.com/${user}" target="_blank">${user}</a></span>${text}</p>');
		html.push('<ul class="info">');
		html.push('<li class="reply"><a href="${tweetReply}" target="_blank">Reply</a></li>');
		html.push('<li class="retweet"><a href="${reTweet}" target="_blank">Retweet</a></li>');
		html.push('<li class="favorite last"><a href="${faveLink}" target="_blank">Favorite</a></li>');
		html.push('</ul>');
		html.push('<span class="time"><a href="${tweetLink}" target="_blank">${tweetTime.str}</a> via ${tweetSource}</span>');
		html.push('<span class="time-obj">${tweetTime.val}</span>');
		html.push('</div><div class="clr-all"></div></li>');
		return (html.join(""));
	}());
	
	window.TwitterAPIPort = function() {
		this.currentXHR = false;
		this.intervalObj = false;
		this.feedInterval = false;
		this.loadInterval = 0;
		this.config = {};
		this.dataObj = {};
		
		// can only refresh every 5 seconds (spam filter)
		this.refreshInterval = 5000; 
		this.refreshAvailable = true;
		
		this.feed = new Feed(this);
		this.render = new Render(this);
		this.parser = new Parser(this);
		this.controls = new Controls(this);
		
	};
	
	TwitterAPIPort.prototype = {
		init: function(config) {
			if (!config) { return false; }
			var TW = this;
			
			for (var i in config) { TW.config["_"+i] = config[i]; }
			if (!TW.config._elm) { return false; }
			if (TW.config._elm && TW.config._elm.size()===0) { return false; }
			
			TW.dataObj.rpp = (TW.config._resultsPerPage && !isNaN(TW.config._resultsPerPage)) ? parseInt(TW.config._resultsPerPage,10) : CONST.defaults.resultsPerPage;
			
			// view
			if (!TW.config._view) { TW.config._view = CONST.defaults.view; }
			
			// stack settings
			if (TW.config._view.type==="stack") {
				TW.config._view.freq = (!isNaN(TW.config._view.freq)) ? parseInt(TW.config._view.freq,10) : CONST.defaults.stackFreq;
				TW.config._view.primary = (TW.config._view.primary) ? TW.config._view.primary : CONST.defaults.stackPrimary;
			}
			
			function getFeed() {
				TW.getFeed({ search:TW.config._search, rpp:TW.dataObj.rpp },TW.config._refresh);
				// set title
				TW.elmObj.title.html(TW.config._search);
			}
			
			TW.setHolders();
			TW.setListeners();
			
			// if trends
			if (TW.config._trend) {
				var cookieVal = TW.cookie.get();
				if (cookieVal) {
					TW.setTrend(cookieVal);
					getFeed();
				} else {
					TW.feed.getTrend(function(){ getFeed(); });
				}
			} else {
				getFeed();
			}
		},
		setHolders: function() {
			var TW = this, elm = TW.config._elm;
			
			TW.elmObj = {
				elm: elm,
				title: elm.find("> .title"),
				listCont: elm.find("> .c-list"),
				list: elm.find("> .c-list .list"),
				refresh: elm.find("> .controls .refresh"),
				pause: elm.find("> .controls .pause")
			};
			
		},
		setListeners: function() {
			var TW = this, elmObj = TW.elmObj, pauseElm = elmObj.pause.find("a"), paused = false;
			
			elmObj.list.hover(function(){ $(this).addClass("hover"); },function(){ $(this).removeClass("hover"); });

			function pause(forced) {
				paused = (typeof forced==="boolean") ? forced : (!paused) ? true : false;
				elmObj.list.toggleClass("pause",paused);
				pauseElm.html((paused) ? "unpause" : "pause");
			}
			
			pauseElm.click(function(){
				pause();
				return false;
			});
			
			elmObj.refresh.find("a").click(function(){
				TW.controls.reload();
				pause(false);
				return false;
			});
		},
		getFeed: function(cfg,refresh) {
			refresh = (refresh) ? true : false;
			var TW = this;
			
			TW.feedInterval = (!refresh) ? false : (isNaN(TW.config._refresh)) ? CONST.defaults.refresh : parseInt(TW.config._refresh,10);
			
			// reset
			TW.loadInterval = 0; 
			TW.render.clearList();
			
			TW.feed.get(cfg,"new");
			
			if (TW.feedInterval && TW.config._view.type==="batch") {
				TW.intervalObj = setInterval(function(){
					TW.feed.refresh();
				},helper.setRefresh(TW.feedInterval,CONST.defaults.minRefresh));
			}
		},
		setTrend: function(str) {
			var TW = this;
			if (typeof str !== "string") { return; }
			if (str.length===0) { return; }
			var arr = str.split("|"), len = arr.length, rand = Math.floor(Math.random()*len),
				search = (arr[rand]) ? arr[rand] : arr[0];
			TW.config._search = search;
		}
	};
	
	TwitterAPIPort.prototype.cookie = { // cookie controls
		get: function() {
			var val = false;
			if ($.cookie) { val = $.cookie(CONST.cookie.name) || false; }
			return val;
		},
		save: function(val) {
			if ($.cookie) {
				var expires = new Date();
				expires.setTime(expires.getTime() + (CONST.cookie.expires * 60 * 1000));
				$.cookie(CONST.cookie.name, val, { expires: expires, path: '/' });
			}
		}
	};
	
	/*** Controls ***/
	function Controls(root) { 
		this.root = root; 
	}
	
	Controls.prototype = {
		stopInterval: function() {
			var root = this.root;
			clearInterval(root.intervalObj);
			root.intervalObj = false;
			if (root.currentXHR) { root.currentXHR.abort(); root.currentXHR = false; }
			log("[TwitterAPIPort] Twitter refresh interval stopped.");
		},
		reload: function() {
			var root = this.root, refreshElm = root.elmObj.refresh;
			
			if (root.refreshAvailable) {
				refreshElm.css("visibility","hidden");
				root.refreshAvailable = false;
				setTimeout(function(){
					root.refreshAvailable = true;
					refreshElm.css("visibility","visible");
				},root.refreshInterval);
				this.stopInterval();
				root.getFeed({ search:root.config._search, rpp:root.dataObj.rpp },root.config._refresh);
				log("[TwitterAPIPort] Twitter feeds reloaded.");
			} else {
				log("[TwitterAPIPort] Reload unavailable.");
			}
		}
	};
	
	/*** Feeds ***/
	function Feed(root) {
		this.root = root;
	}
	
	Feed.prototype = {
		get: function(cfg,type) {
			var TW = this, root = this.root, feedObj = CONST.feed; 
			
			var url = (function(){
				var val = "";
				
				if (type==="refresh" && root.dataObj.refreshUrl) {
					val += root.dataObj.refreshUrl;
					// if stack, use the maximum allowable results per page by twitter
					if (root.config._view.type!=="stack") { val += "&rpp=" + cfg.rpp; }
					val += "&callback=?";
				} else {
					val += feedObj.query;
					val = val.replace("${search}",encodeURIComponent(cfg.search));
					// if stack and single rendering, use the maximum allowable results per page by twitter
					val = val.replace("${rpp}",(root.config._view.type==="stack" && root.config._view.primary==="single") ? "15" : cfg.rpp);
					
					val = val.replace("${refresh}","");
				}
				
				val = val.replace("${callback}","?");
				val = feedObj.url + val;
				
				var uri, query, idx = val.indexOf("?")+1;
				uri = val.slice(0,idx);
				query = val.slice(idx,val.length);
				return uri+query;
			}());
			
			// try to abort xmlhttprequest
			try { if (root.currentXHR) { root.currentXHR.abort(); } } catch(err) {}
			
			function trigger() {
				root.currentXHR = $.ajax({
					url: url,
					dataType:feedObj.type,
					success: function(raw) {
						TW.callback(raw);
					},
					error: function(status) {
						// fail silently
						root.currentXHR = false;
						root.loadInterval++;
						log("[TwitterAPIPort] Request Error @ load interval #"+root.loadInterval+": " + status.statusText);
						log("[TwitterAPIPort] Attempting to Re-trigger...");
						
						// attempt again
						setTimeout(function(){ 
							trigger(); 
						},3000);
					}
				});
			}
			
			trigger();
		},
		callback: function(raw) {
			var TW = this, root = this.root, listCont = root.elmObj.listCont;
			
			root.currentXHR = false;
			var parsed = root.parser.tweets(raw);
			TW.set(parsed);
			if (root.config._view.type==="stack") {
				root.render.listStack(parsed);
			} else {
				root.render.list(parsed);
			}
			if (!listCont.hasClass("loaded")) {
				listCont.addClass("loaded");
			}
			root.loadInterval++;
		},
		getTrend: function(callback) {
			var TW = this, root = this.root, hasTriggered = false;
			
			function trigger() {
				if (typeof callback==="function" && !hasTriggered) { hasTriggered = true; callback(); }
			}
			
			function setTrend(data,type) {
				var x, i, str = [];
				
				if (data && data.trends) {
					var obj = data.trends, query;
					query = {};
					for (i in obj) { query = obj[i]; break; }
					for (x = 0; x < query.length; x++) { str.push(query[x].query); }
					str = str.join("|");
					root.cookie.save(str);
					root.setTrend(str);
				}
				
				trigger();
			}
			
			var url = CONST.trend.url, trend = root.config._trend.toLowerCase();
			
			switch(trend) {
				case "current": url = url.replace("${format}","current"); break;
				case "daily": url = url.replace("${format}","daily"); break;
				default: url = false; // use custom search
			}
			
			if (!url) {
				setTrend(root.config._search,"search");
				trigger();
			} else {
				root.currentXHR = $.ajax({
					url: url,
					data: CONST.trend.data || {},
					dataType: CONST.trend.type,
					success: function(data) {
						setTrend(data,trend);
					},
					error: function() {
						trigger();
					}
				});
			}
			
			var reTrigger = setTimeout(function(){
				if (!hasTriggered) {
					setTrend(root.config._search,"search");
					log("[TwitterAPIPort] Your IP has been flagged for rate limiting.. using default search results");
				}
			},3000);
		},
		set: function(data) {
			var TW = this, root = this.root;
			
			// set refresh url
			root.dataObj.refreshUrl = (data.refresh_url) ? data.refresh_url : false;
		},
		refresh: function() {
			var TW = this, root = this.root;
			TW.get({ search:root.config._search, rpp:root.dataObj.rpp },"refresh");
		}
	};
	
	/*** Render ***/
	function Render(root) {
		this.root = root;
	}
	
	Render.prototype = {
		listStack: function(data) {
			if (data && data.error) { log("[TwitterAPIPort] Feed error: " + data.error);return false;}
			
			var x, TW = this, root = this.root, results = data.results, len = results.length,
				userStack = [], listElm = root.elmObj.list, currentTime = getCurrentTime(), 
				listCont = root.elmObj.listCont, contH = listCont.height() - 20,
				stackFreq = helper.setRefresh(root.config._view.freq,CONST.defaults.stackFreq);
			
			for (x = 0; x < len; x++) {
				var item = results[x];
				userStack.push(TW.insertTweet(item,currentTime,true));
			}
			
			function stackTimeout() {
				root.intervalObj = setTimeout(function(){
					stackItUp();
				},stackFreq);
			}
			
			function stackItUp() {
				if (root.intervalObj) {
					clearTimeout(root.intervalObj);
					clearInterval(root.intervalObj);
				}
				if (userStack.length>0) {
					if (!listElm.hasClass("hover") && !listElm.hasClass("pause") && !listElm.hasClass("focused")) {
						var item = userStack[userStack.length-1];
						animateStackItem(item);
						TW.updateList(currentTime);
						userStack.pop();
					}
					stackTimeout();
				} else {
					root.intervalObj = false;
					root.feed.refresh();
				}
			}
			
			function postEventListener(elm) {
				var id = elm.attr("id");
				
				// show the full item if it is cut off at the bottom
				elm.hover(function(){
					var viewH = 0;

					listElm.find("> li").each(function(){
						var item = $(this), iid = item.attr("id");
						viewH += item.outerHeight();
						if (iid===id) { return false; }
					});

					if (viewH >= contH && !listElm.hasClass("focused")) {
						listElm.addClass("focused");
						var diff = (viewH - contH) + 10;
						elm.data("_diff",diff);
						listCont.animate({ scrollTop:diff },200,"swing");
					}
				},function(){
					if (listElm.hasClass("focused") && elm.data("_diff") && elm.data("_diff") > 0) {
						elm.data("_diff",0);
						listCont.animate({ scrollTop:0 },200,"swing",function(){
							listElm.removeClass("focused");
						});
					}
				});
			}
			
			function animateStackItem(item) {
				item = $(item);
				listElm.prepend(item);
				var elm = listElm.find("> li:first");
				preloadImg(elm.find(".user-img img"));
				elm.css({ opacity:0, backgroundColor:"#fff" });
				elm.slideDown("slow",function(){
					elm.animate({ opacity:1 },200,"linear",function(){
						postEventListener(elm);
					}).css({ filter:"" });
				});
			}
			
			if (root.config._view.primary==="single" && userStack.length>0) {
				stackItUp();
			} else if (root.loadInterval===0 || userStack.length===0) {
				if (userStack.length>0) {
					listElm.html($(userStack.join(""))); 
					listElm.find("> li").show();
				}
				root.intervalObj = setTimeout(function(){
					root.feed.refresh();
					clearTimeout(root.intervalObj);
					root.intervalObj = false;
				},helper.setRefresh(root.feedInterval,CONST.defaults.minRefresh));
				
				TW.updateList(currentTime);
			} else {
				userStack.reverse(); // reverse in prep for pop() - refresh requests needs reverse
				stackItUp();
			}
		},
		list: function(data) {
			if (data && data.error) { log("[TwitterAPIPort] Feed error: " + data.error);return false;}
			
			var x, TW = this, root = this.root, results = data.results, len = results.length,
				userHtml = [], listElm = root.elmObj.list, currentTime = getCurrentTime();

			for (x = 0; x < len; x++) {
				var item = results[x];
				userHtml.push(TW.insertTweet(item,currentTime));
			}
			var users = $(userHtml.join(""));
			
			if (root.loadInterval===0) {
				listElm.html(users);
			} else {
				listElm.prepend(users);
			}
			preloadImg(listElm.find("> li:lt("+len+") .user-img img"));
			TW.updateList(currentTime);
		},
		updateList: function(currentTime) {
			var TW = this, root = this.root, listElm = root.elmObj.list, items = listElm.children();
			if (items.size()===0) { return; } // do nothing if there are no items
			
			currentTime = currentTime || getCurrentTime();
			var max = (!isNaN(root.config._maxItems)) ? parseInt(root.config._maxItems,10) : CONST.defaults.maxItems;
			
			// remove excess items
			listElm.children(":gt("+(max-1)+")").remove();
			// re-declare with the updated children
			items = listElm.children();
			var last = items.size()-1;
			
			items.each(function(i){
				var item = $(this), content = item.find(">.content");
				// update <li> class
				item.removeClass("first last");
				if (i===0 && last!==0) { 
					item.addClass("first"); 
				} else if (i===last) { 
					item.addClass("last"); 
				}
				
				// update time stamps
				var display = content.find("> .time a:first"),
					raw = content.find("> .time-obj"),
					displayTxt = display.text();
					
				if (raw.size()>0) {
					var tweetTime = TW.timeStr(raw.text(),currentTime);
					if (displayTxt!==tweetTime.str) { display.html(tweetTime.str); }
				}	
			});
			
			// if there is a special callback per update
			if (typeof root.config._onUpdateCallback === "function") {
				root.config._onUpdateCallback(items);
			}
			
		},
		clearList: function() {
			var TW = this, root = this.root;
			root.elmObj.list.children().remove();
		},
		insertTweet: function(item,currentTime,isHidden) {
			isHidden = isHidden || false;
			
			var TW = this, root = this.root,
				markup = (root.config && root.config._markup) ? root.config._markup : CONST.defaultMarkup,
				baseUrl = "http://twitter.com/",
				user = item.from_user,
				userId = item.from_user_id,
				text = item.text,
				imgSrc = item.profile_image_url,
				dateStr = item.created_at,
				source = item.source,
				tweetId = item.id_str,
				tweetTime = TW.timeStr(dateStr,currentTime),
				tweetSource = helper.setTags(source),
				tweetLink = baseUrl + user + "/status/" + tweetId,
				tweetReply = baseUrl + "intent/tweet?in_reply_to=" + tweetId,
				// tweetReply = baseUrl + "/?status=@" + user + "&in_reply_to_status_id=" + tweetId
				reTweet = baseUrl + "intent/retweet?tweet_id=" + tweetId,
				faveLink = baseUrl + "intent/favorite?tweet_id=" + tweetId;
				
				// http://twitter.com/intent/tweet?in_reply_to=56432614793416705

			// set up as off page link
			tweetSource = tweetSource.replace("<a",'<a target="_blank"');

			// set up text message
			text = TW.setMessage(text);
			
			var hideStyle = (isHidden) ? ' class="hide"' : '', html = [];
			
			// replace
			var replace = {
				"${hideStyle}": hideStyle,
				"${tweetId}": tweetId,
				"${user}": user,
				"${imgSrc}": imgSrc,
				"${tweetReply}": tweetReply,
				"${reTweet}": reTweet,
				"${faveLink}": faveLink,
				"${text}": text,
				"${tweetLink}": tweetLink,
				"${tweetTime.str}": tweetTime.str,
				"${tweetSource}": tweetSource,
				"${tweetTime.val}": tweetTime.val
			};
			
			for (i in replace) {
	   		var idx = markup.indexOf(i);
	   		while (idx!==-1) {
	   			markup = markup.replace(i,replace[i]);
	   			idx = markup.indexOf(i);
	   		}
	   	}
			
			return markup;
		},
		timeStr: function(val,currentTime) {
			var TW = this, root = this.root,
				time = helper.setDate(val);
				
			if (!time) { return { str:"",val:false }; }
			
			var tweetTime = time.getTime(),
				now = currentTime.getTime(),
				diff = Math.round((now - tweetTime) / 1000), // in seconds
				min = diff / 60,
				hr = min / 60,
				day = hr / 24;
			
			var str = "";
			
			if (min < 0) {
				str = "&nbsp;";
			} else if (min < 2) {
				if (diff === 1) {
					str = "a second ago";
				} else if (diff < 60) {
					str = diff + " seconds ago";
				} else {
					str = "1 minute ago";
				}
			} else if (hr < 2) {
				if (min < 60) {
					str = Math.floor(min) + " minutes ago";
				} else {
					str = "1 hour ago";
				}
			} else if (day < 2) {
				if (hr < 24) { 
					str = Math.floor(hr) + " hours ago";
				} else {
					str = "1 day ago";
				}
			} else if (day < 32) {
				if (day < 31) {
					str = Math.floor(day) + " days ago";
				} else {
					str = "a month ago";
				}
			} else {
				str = "eons ago...";
			}

			var obj = { str:str,val:val };
			return obj;
		},
		setMessage: function(text) {
			var x, i, TW = this;
			var matches = {
				links: text.match(/\http:\/\/\S+/gi),
				users: text.match(/@[a-zA-Z0-9\-\_]+/gi),
				terms: text.match(/#[a-zA-Z0-9\-\_]+/gi)
			};

			var replace = {
				links: '<a target="_blank" href="${item}" target="_blank" class="hyper-word">${item}</a>',
				users: '<a target="_blank" href="http://twitter.com/${link}">${item}</a>',
				terms: '<a target="_blank" href="http://twitter.com/${link}">${item}</a>'
			};

			for (i in matches) {
				var match = matches[i];
				if (match) {
					var len = match.length;
					for (x = 0; x < len; x++) {
						var item = match[x];
						if (i==="users") {
							text = text.replace(item,replace[i].replace(/(\$\{item\})/g,item).replace(/(\$\{link\})/g,item.slice(1)));
						} else if (i==="terms") {
							text = text.replace(item,replace[i].replace(/(\$\{item\})/g,item).replace(/(\$\{link\})/g,"search?q="+item.slice(1)));
						}
						else {
							text = text.replace(item,replace[i].replace(/(\$\{item\})/g,item));
						}
					}
				}
			}

			return text;
		}
	};
	
	/*** Parser ***/
	function Parser(root) {
		this.root = root;
	}
	
	Parser.prototype = {
		tweets: function(data) {
			return data;
		}
	};
	
	var helper = {
		setRefresh: function(val,defVal) {
			val = (isNaN(val)) ? CONST.defaults.refresh : parseInt(val,10);
			val = (val < defVal) ? defVal : val;
			val = val * 1000;
			return val;
		},
		setDate: function(val) {
			try {
				var date = new Date(val);
				return (!isNaN(date.getTime())) ? date : false;
			} catch(err) {
				return false;
			}
		},
		setTags: function(str) {
			str = unescape(str);
			str = str.replace(/\&lt;/gi, "<").replace(/\&gt;/gi, ">").replace(/\&quot;/gi,"\"");

			return str;
		}
	};
	
	function getCurrentTime() {
		return (new Date());
	}
	
	function preloadImg(elm) {
		elm.each(function(){
			var item = $(this), img = new Image();
			item.css({ opacity:0 });
			img.onload = function() {
				item.addClass("loaded");
				item.animate({ opacity:1 }, 150, "linear", function(){
					if ($.browser.msie) { item.css("filter",""); }
				});
			};
			img.src = item.attr("src");
			setTimeout(function(){
				if (!item.hasClass("loaded")) { item.css({ opacity:"1", filter:"" }); }
			},3000);
		});		
	}
	
	function log(str) {
		if (window.console && console.log) { console.log(str); }
	}
	
}(jQuery));
